"""Console entry point for the LeGen CLI."""
from __future__ import annotations

import sys
from typing import Sequence

from legen import main as _run_main


def main(argv: Sequence[str] | None = None) -> None:
    """Run the packaged LeGen CLI and propagate its exit code."""
    exit_code = _run_main(argv)
    if exit_code not in (None, 0):
        sys.exit(exit_code)


if __name__ == "__main__":  # pragma: no cover
    main()



import atexit
import os
import re
import tkinter as tk
import tkinter.font as tk_font
from pathlib import Path

import pysrt

_STRING_WIDTH_ROOT = None
_FONT_CACHE = {}


def _destroy_string_width_root():
    global _STRING_WIDTH_ROOT
    if _STRING_WIDTH_ROOT is None:
        return
    try:
        _STRING_WIDTH_ROOT.destroy()
    except Exception:
        pass
    _STRING_WIDTH_ROOT = None


def _ensure_string_width_font(font_name: str, font_size: int) -> tk_font.Font:
    global _STRING_WIDTH_ROOT
    if _STRING_WIDTH_ROOT is None:
        _STRING_WIDTH_ROOT = tk.Tk()
        _STRING_WIDTH_ROOT.withdraw()
        atexit.register(_destroy_string_width_root)

    cache_key = (font_name, font_size)
    font = _FONT_CACHE.get(cache_key)
    if font is None:
        font = tk_font.Font(root=_STRING_WIDTH_ROOT, family=font_name, size=font_size, weight="bold")
        _FONT_CACHE[cache_key] = font
    return font


def _collect_plain_text(subtitles: pysrt.SubRipFile) -> str:
    texts = []
    for item in subtitles:
        normalized = " ".join(item.text.strip().split())
        if normalized:
            texts.append(normalized)
    return " ".join(texts).strip()


def export_plain_text_from_srt(source, output_path: Path) -> str:
    """Save subtitles as a single-line TXT without timestamps or line breaks."""
    if isinstance(source, pysrt.SubRipFile):
        subtitles = source
    else:
        subtitles = pysrt.open(source, encoding="utf-8")

    plain_text = _collect_plain_text(subtitles)

    os.makedirs(output_path.parent, exist_ok=True)
    with open(output_path, "w", encoding="utf-8") as f:
        f.write(plain_text)

    return plain_text


def SaveSegmentsToSrt(segments: list, output_path: Path):
    # Create the subtitle file
    subs = pysrt.SubRipFile()
    sub_idx = 1

    for i in range(len(segments)):
        start_time = segments[i]["start"]
        end_time = segments[i]["end"]
        duration = end_time - start_time
        timestamp = f"{start_time:.3f} - {end_time:.3f}"
        text = segments[i]["text"]

        sub = pysrt.SubRipItem(index=sub_idx, start=pysrt.SubRipTime(seconds=start_time),
                               end=pysrt.SubRipTime(seconds=end_time), text=text)
        subs.append(sub)
        sub_idx += 1

    # make dir and save .srt
    os.makedirs(output_path.parent, exist_ok=True)
    subs.save(output_path)


def string_width(text, font_name="Jost", font_size=18):
    """
    Determines the width of a string using tkinter.
    """
    tries_remaining = 5
    
    while (tries_remaining > 0):
        tries_remaining -= 1
        try:
            font = _ensure_string_width_font(font_name, font_size)
            width = font.measure(text)
            return width
        except Exception:
            _destroy_string_width_root()
            pass

    # all failed, return 60% of height per char
    return len(text) * font_size * 0.60


def is_punctuation_end(word):
    """Verifica se a palavra termina com uma pontuação."""
    return any(word.endswith(punct) for punct in ['.', ',', '!', '?', ':', ';'])


def split_segments(segments, max_width_px=1440, font_name="Jost", font_size=18):
    """
    Split segments based on the max width provided.
    """
    new_segments = []
    for segment in segments:
        words = segment['words']
        current_words = []
        current_width = 0

        for word in words:
            # Calculate the width with a space after the word
            added_width = string_width(
                word['word'] + " ", font_name, font_size)
            isolated_sentence_ending = is_punctuation_end(word['word']) and not (
                current_words and is_punctuation_end(current_words[-1]['word']))
            possible_logical_break_point = len(current_words) >= 2 and len(
                current_words[-1]['word']) <= 3 and not len(current_words[-2]['word']) <= 3

            if (current_width + added_width < max_width_px) or len(current_words) == 0 or isolated_sentence_ending or possible_logical_break_point:
                current_words.append(word)
                current_width += added_width
            else:
                new_segments.append({
                    'text': ' '.join(word['word'] for word in current_words),
                    'start': next((word['start'] for word in current_words if 'start' in word), segment['start']),
                    'end': next((word['end'] for word in reversed(current_words) if 'end' in word), segment['end']),
                    'words': current_words.copy()
                })
                current_words = [word]
                current_width = added_width

        # For any remaining words
        if current_words:
            new_segments.append({
                'text': ' '.join(word['word'] for word in current_words),
                'start': next((word['start'] for word in current_words if 'start' in word), segment['start']),
                'end': next((word['end'] for word in reversed(current_words) if 'end' in word), segment['end']),
                'words': current_words
            })

    return new_segments


def split_string_to_max_lines(text, max_width=720, max_lines=2, font_name="Jost", font_size=18):
    threshold = max_width * 0.8
    total_text_width = string_width(text, font_name, font_size)

    if total_text_width <= threshold or max_lines < 2:
        return [text]

    words = text.split()
    lines = []
    current_line_words = []
    current_line_width = 0

    for i, word in enumerate(words):
        word_width = string_width(word + ' ', font_name, font_size)
        isolated_sentence_ending = is_punctuation_end(word) and not (
            current_line_words and is_punctuation_end(current_line_words[-1]))
        possible_logical_break_point = len(current_line_words) >= 2 and len(
            current_line_words[-1]) <= 3 and not len(current_line_words[-2]) <= 3

        if current_line_width + word_width < total_text_width / max_lines or len(current_line_words) == 0 or isolated_sentence_ending or possible_logical_break_point:
            current_line_words.append(word)
            current_line_width += word_width
        else:
            lines.append(' '.join(current_line_words))
            current_line_words = [word]
            current_line_width = word_width

        if len(lines) == max_lines - 1:
            remaining_words = words[i:]
            lines.append(' '.join(remaining_words))
            break

    if current_line_words and len(lines) < max_lines:
        lines.append(' '.join(current_line_words))

    return lines


def adjust_times(segments, extra_end_time=1.0):
    for i in range(len(segments) - 1):  # We don't need to check the last segment
        current_end = segments[i]['end']
        next_start = segments[i + 1]['start']

        gap = next_start - current_end

        # If the gap is more than 1.5 + extra_end_time
        if gap > 1.5 + extra_end_time:
            segments[i]['end'] = current_end + extra_end_time

        # If the gap is less than 1.5 + extra_end_time
        elif gap < 1.5 + extra_end_time:
            segments[i]['end'] = next_start

    return segments


def format_segments(segments: list, max_line_width_px: int = 380, max_lines_per_segment: int = 2):
    print('Formatting segments...', end='', flush=True)

    segments = split_segments(
        segments, max_line_width_px * max_lines_per_segment)

    for segment in segments:
        segment["text"] = "\n".join(split_string_to_max_lines(
            text=segment["text"], max_width=max_line_width_px, max_lines=max_lines_per_segment))

    segments = adjust_times(segments)
    
    print('\r                      ', end='\r', flush=True)

    return segments



import tempfile
import unittest
from pathlib import Path

import utils


class UtilsTests(unittest.TestCase):
    def test_format_time_formats_components(self):
        self.assertEqual(utils.format_time(3661), "1h 1m 1s")
        self.assertEqual(utils.format_time(0), "0s")

    def test_check_other_extensions_finds_matches(self):
        with tempfile.TemporaryDirectory() as tmpdir:
            tmp_path = Path(tmpdir)
            base = tmp_path / "video.mp4"
            sibling = tmp_path / "video.mkv"
            unrelated = tmp_path / "other.mp4"

            base.write_text("base", encoding="utf-8")
            sibling.write_text("sibling", encoding="utf-8")
            unrelated.write_text("unrelated", encoding="utf-8")

            matches = utils.check_other_extensions(base, [".mp4", ".mkv"])
            self.assertIn(sibling, matches)
            self.assertIn(base, matches)
            self.assertNotIn(unrelated, matches)


if __name__ == "__main__":  # pragma: no cover
    unittest.main()



import importlib
import unittest

try:
    translate_utils = importlib.import_module("translate_utils")
except ModuleNotFoundError:  # pragma: no cover - dependency chain might be missing
    translate_utils = None


@unittest.skipIf(translate_utils is None, "translate_utils dependencies are unavailable")
class TranslateUtilsTests(unittest.TestCase):
    def test_join_sentences_respects_sentence_endings(self):
        lines = ["Hello world.", "Another line!", "Trailing"]
        chunks = translate_utils.join_sentences(lines, max_chars=200)

        self.assertTrue(chunks)
        for chunk in chunks:
            self.assertLessEqual(len(chunk), 200)
            self.assertTrue(chunk.endswith(translate_utils.separator))

    def test_unjoin_sentences_preserves_line_count(self):
        original_lines = ["Hello world.", "Another line."]
        original_chunk = translate_utils.join_sentences(original_lines, max_chars=200)[0]
        modified_chunk = translate_utils.separator_unjoin.join(["Hola mundo.", "Otra linea."])

        rebuilt = translate_utils.unjoin_sentences(original_chunk, modified_chunk, translate_utils.separator_unjoin)
        self.assertEqual(len(rebuilt), len(original_lines))
        self.assertTrue(all(isinstance(item, str) for item in rebuilt))

    def test_unjoin_sentences_handles_missing_translation(self):
        original_lines = ["Only line."]
        original_chunk = translate_utils.join_sentences(original_lines, max_chars=200)[0]
        rebuilt = translate_utils.unjoin_sentences(original_chunk, None, translate_utils.separator_unjoin)
        self.assertEqual(rebuilt, original_lines or ' ')


if __name__ == "__main__":  # pragma: no cover
    unittest.main()



import importlib
import tempfile
import unittest
from pathlib import Path
from unittest.mock import patch

try:
    import pysrt
except ModuleNotFoundError:  # pragma: no cover - dependency might be missing in test envs
    pysrt = None

try:
    subtitle_utils = importlib.import_module("subtitle_utils")
except ModuleNotFoundError:  # pragma: no cover - dependency chain might be missing
    subtitle_utils = None


@unittest.skipIf(subtitle_utils is None or pysrt is None, "subtitle_utils dependencies are unavailable")
class SubtitleUtilsTests(unittest.TestCase):
    def test_export_plain_text_from_srt_writes_single_line(self):
        subs = pysrt.SubRipFile()
        subs.append(pysrt.SubRipItem(index=1, text="Hello\nworld", start=pysrt.SubRipTime(seconds=0), end=pysrt.SubRipTime(seconds=1)))
        subs.append(pysrt.SubRipItem(index=2, text="Another line", start=pysrt.SubRipTime(seconds=1), end=pysrt.SubRipTime(seconds=2)))

        with tempfile.TemporaryDirectory() as tmpdir:
            output_path = Path(tmpdir) / "out.txt"
            text = subtitle_utils.export_plain_text_from_srt(subs, output_path)

            self.assertEqual(text, "Hello world Another line")
            self.assertEqual(output_path.read_text(encoding="utf-8"), text)

    @patch("subtitle_utils.string_width", side_effect=lambda text, *_: len(text) * 100)
    def test_split_segments_respects_max_width(self, mock_width):
        segments = [{
            "text": "Hello brave new world",
            "start": 0.0,
            "end": 4.0,
            "words": [
                {"word": "Hello", "start": 0.0, "end": 0.5},
                {"word": "brave", "start": 0.5, "end": 1.0},
                {"word": "new", "start": 1.0, "end": 1.5},
                {"word": "world", "start": 1.5, "end": 2.0},
            ],
        }]

        result = subtitle_utils.split_segments(segments, max_width_px=200)
        self.assertGreaterEqual(len(result), 2)
        combined = " ".join(segment["text"].replace("\n", " ") for segment in result)
        self.assertIn("Hello brave", combined)

    @patch("subtitle_utils.string_width", side_effect=lambda text, *_: len(text) * 10)
    def test_split_string_to_max_lines_balances_lines(self, mock_width):
        lines = subtitle_utils.split_string_to_max_lines("one two three four five", max_width=30, max_lines=2)
        self.assertLessEqual(len(lines), 2)
        self.assertEqual(" ".join(lines).replace("  ", " ").strip(), "one two three four five")

    def test_adjust_times_expands_large_gaps(self):
        segments = [
            {"start": 0.0, "end": 0.5},
            {"start": 4.0, "end": 5.0},
        ]
        adjusted = subtitle_utils.adjust_times([dict(seg) for seg in segments], extra_end_time=1.0)
        self.assertAlmostEqual(adjusted[0]["end"], 1.5)

    def test_adjust_times_clamps_small_gaps(self):
        segments = [
            {"start": 0.0, "end": 0.5},
            {"start": 1.0, "end": 2.0},
        ]
        adjusted = subtitle_utils.adjust_times([dict(seg) for seg in segments], extra_end_time=1.0)
        self.assertEqual(adjusted[0]["end"], segments[1]["start"])


if __name__ == "__main__":  # pragma: no cover
    unittest.main()



import os
from pathlib import Path

import pysrt
import whisper
import whisper.transcribe
import whisperx_legen_fork as whisperx
from whisperx_legen_fork import alignment, asr, utils, audio
import subtitle_utils
from utils import time_task


def transcribe_audio(model: whisper.model, audio_path: Path, srt_path: Path, lang: str = None, disable_fp16: bool = False):
    # Load audio
    audio = whisper.load_audio(file=audio_path.as_posix())
    
    # Transcribe
    with time_task():
        transcribe = model.transcribe(audio=audio, language=lang, fp16=False if disable_fp16 else True, verbose=False)

    # Align if possible
    if lang in alignment.DEFAULT_ALIGN_MODELS_HF or lang in alignment.DEFAULT_ALIGN_MODELS_TORCH:
        with time_task(message_start="Running alignment..."):
            try:
                model_a, metadata = alignment.load_align_model(language_code=lang, device="cuda")
                transcribe = alignment.align(transcript=transcribe["segments"], model=model_a, align_model_metadata=metadata, audio=audio, device="cuda", return_char_alignments=True)
            except Exception:
                model_a, metadata = alignment.load_align_model(language_code=lang, device="cpu")  # force load on cpu due errors on gpu
                transcribe = alignment.align(transcript=transcribe["segments"], model=model_a, align_model_metadata=metadata, audio=audio, device="cpu", return_char_alignments=True)
    else:
        print(f"Language {lang} not suported for alignment. Skipping this step")

    # Format subtitles
    segments = subtitle_utils.format_segments(transcribe['segments'])

    # Save the subtitle file
    subtitle_utils.SaveSegmentsToSrt(segments, srt_path)

    return transcribe


def detect_language(model: str, audio_path: Path):
    # load audio and pad/trim it to fit 30 seconds
    audio = whisper.load_audio(audio_path.as_posix())
    audio = whisper.pad_or_trim(audio)
    # make log-Mel spectrogram and move to the same device as the model
    mel = whisper.log_mel_spectrogram(audio).to(model.device)

    # detect the spoken language
    _, probs = model.detect_language(mel)
    return max(probs, key=probs.get)
